from colored import fg, attr
# color chart see https://pypi.org/project/colored/


class Printer:
    """ Utility to print colorful text """

    def __init__(self, color=True):
        # color: set to False to diable colors
        self._color = color

    def _colorprint(self, text, fgc):
        if self._color:
            print(f'{fg(fgc)}{text}{attr(0)}')
        else:
            print(text)

    def black(self, t):
        self._colorprint(t, 0)

    def red(self, t):
        self._colorprint(t, 1)

    def green(self, t):
        self._colorprint(t, 2)

    def yellow(self, t):
        self._colorprint(t, 3)

    def blue(self, t):
        self._colorprint(t, 4)

    def magenta(self, t):
        self._colorprint(t, 5)

    def cyan(self, t):
        self._colorprint(t, 6)

    def gray(self, t):
        self._colorprint(t, 7)

    def white(self, t):
        self._colorprint(t, 15)



from argparse import ArgumentParser
import sys
import os
import textwrap
from wiktionaryparser import WiktionaryParser
from .utils import *
from .fancyprint import Printer
import colored


def main():
    wikt = WiktionaryParser()

    argparser = ArgumentParser()
    argparser.add_argument('word', help='word to look up')
    argparser.add_argument('-d', '--definition',
                           action='store_true', help='print definition only')
    argparser.add_argument(
        '--no-color', action='store_true', default=False,
        help='disable color printing (when stdout is not a tty this is automatically on)')
    args = argparser.parse_args()

    # only use colors when user did not specify --no-color 
    # and the output is printed in a tty, i.e. console/terminal,
    # not a file/pipe.
    color_on = (not args.no_color) and sys.stdout.isatty()
    printer = Printer(color_on)

    try:
        etymologies = wikt.fetch(args.word)
        # an example etymologies dict for debug use is in ../example.py
    except KeyboardInterrupt:
        printer.red('\nAborted')
        sys.exit(1)

    for ety_no, etymology in enumerate(etymologies, 1):
        if not args.definition:
            printer.blue(f'Etymology {ety_no}:')
            printer.white(leftpad(etymology['etymology'], 4))
            print()

        printer.cyan(leftpad('Definitions:', 4))

        for definition in etymology['definitions']:
            printer.white(leftpad('({0}) {1}'.format(
                # print PoS in green if --no-color is not present
                (colored.fg(2) + definition['partOfSpeech'] + colored.attr(0)
                    if color_on else definition['partOfSpeech']),
                definition['text'][0]), 8))

            for line_no, line in enumerate(definition['text'][1:], 1):
                printer.white(leftpad(str(line_no) + '. ' + line, 12))

            print()

            if definition['examples'] and not args.definition:
                printer.yellow(leftpad('Examples:', 12))
                for ex_no, example in enumerate(definition['examples'], 1):
                    printer.white(leftpad(str(ex_no) + '. ' + example, 16))

        print()


if __name__ == '__main__':
    main()



import os

try:
    term_width = os.get_terminal_size()[0]
except OSError:
    term_width = 80

def leftpad(s: str, n: int, w=term_width) -> str:
    # s is the string, n is indent, w is max width including whitespace
    return '\n'.join([' ' * n + l for l in softwrap(s, w - n)])


def softwrap(s: str, width: int) -> list:
    words = s.split()
    wrapped = []
    line = ''
    for w in words:
        if len(line) + 1 + len(w) <= width:
            line += w + ' '
        else:
            wrapped.append(line)
            line = w + ' '
    wrapped.append(line)
    return wrapped



from setuptools import setup, find_packages

setup(
    name='wikt-cli',
    version='0.0.2',
    description='An English Wiktionary frontend for CLI.',
    long_description=open('./README.md').read(),
    long_description_content_type='text/markdown',
    keywords='wiktionary dictionary cli',
    license='MIT',
    url='https://git.sr.ht/~fkfd/wikt-cli',
    author='fakefred',
    author_email='fkfd@macaw.me',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License'
    ],
    packages=find_packages(),
    entry_points={
        'console_scripts': [
            'wikt=wikt.__main__:main'
        ]
    },
    python_requires='>=3.5',
    install_requires=['wiktionaryparser', 'colored'],
    project_urls={
        'LiberaPay': 'https://liberapay.com/fakefred/donate'
    }
)
